==> exponent_run-0.0.27-py3-none-any.whl: exponent/commands/common.py <==
import _thread
import asyncio
import logging
import os
import sys
from collections.abc import Coroutine
from typing import Any

import click
import httpx
from dotenv import load_dotenv
from exponent.core.config import (
    Settings,
    get_settings,
)
from exponent.core.remote_execution.client import RemoteExecutionClient
from exponent.core.remote_execution.exceptions import ExponentError
from exponent.core.remote_execution.types import (
    RemoteExecutionRequestType,
    RemoteExecutionResponse,
    UseToolsConfig,
)

load_dotenv()


def set_log_level() -> None:
    settings = get_settings()
    logging.basicConfig(level=getattr(logging, settings.log_level), stream=sys.stdout)


def redirect_to_login(settings: Settings, cause: str = "detected") -> None:
    if inside_ssh_session():
        click.echo(f"No API Key {cause}, run 'exponent login --key <API-KEY>'")
    else:
        click.echo("No API Key detected, redirecting to login...")
        click.launch(f"{settings.base_url}/")


def inside_ssh_session() -> bool:
    return (os.environ.get("SSH_TTY") or os.environ.get("SSH_TTY")) is not None


def run_until_complete(coro: Coroutine[Any, Any, Any]) -> Any:
    loop = asyncio.get_event_loop()
    task = loop.create_task(coro)

    try:
        loop.run_until_complete(task)
    except KeyboardInterrupt:
        task.cancel()
        try:
            loop.run_until_complete(task)
        except asyncio.CancelledError:
            pass
    except ExponentError as e:
        click.secho(f"Encountered error: {e}", fg="red")
        click.secho(
            "The Exponent team has been notified, "
            "please try again and reach out if the problem persists.",
            fg="yellow",
        )
        sys.exit(1)


async def benchmark_mode_exit(client: RemoteExecutionClient, chat_uuid: str) -> None:
    while True:
        await asyncio.sleep(5)
        if await client.check_remote_end_event(chat_uuid):
            sys.exit(0)


async def heartbeat_thread(client: RemoteExecutionClient, chat_uuid: str) -> None:
    await client.send_heartbeats(chat_uuid)


async def run_execution_client(client: RemoteExecutionClient, chat_uuid: str) -> None:
    async def handle_request(
        execution_request: RemoteExecutionRequestType,
    ) -> RemoteExecutionResponse:
        execution_response = await client.handle_request(execution_request)
        return execution_response

    await client.for_each_execution_request(chat_uuid, handle_request)


async def signal_listener_thread(client: RemoteExecutionClient, chat_uuid: str) -> None:
    await client.wait_for_disconnect_signal(chat_uuid)
    click.secho("Disconnect signaled. Exiting.")
    _thread.interrupt_main()


async def start_chat(
    client: RemoteExecutionClient,
    chat_uuid: str,
    prompt: str,
    use_tools_config: UseToolsConfig,
) -> None:
    click.secho("Starting chat...")
    await client.start_chat(chat_uuid, prompt, use_tools_config=use_tools_config)
    click.secho("Chat started. Open the link to join the chat.")


async def start_client(
    api_key: str,
    base_api_url: str,
    chat_uuid: str,
    prompt: str | None = None,
    benchmark: bool = False,
) -> None:
    if benchmark is True and prompt is None:
        click.secho("Error: Benchmark mode requires a prompt.", fg="red")
        return

    current_working_directory = os.getcwd()

    use_tools_config = UseToolsConfig()
    async with RemoteExecutionClient.session(
        api_key, base_api_url, current_working_directory
    ) as client:
        if benchmark:
            assert prompt is not None
            await asyncio.gather(
                start_chat(
                    client, chat_uuid, prompt, use_tools_config=use_tools_config
                ),
                run_execution_client(client, chat_uuid),
                benchmark_mode_exit(client, chat_uuid),
                heartbeat_thread(client, chat_uuid),
                signal_listener_thread(client, chat_uuid),
            )
        elif prompt:
            await asyncio.gather(
                start_chat(
                    client, chat_uuid, prompt, use_tools_config=use_tools_config
                ),
                run_execution_client(client, chat_uuid),
                heartbeat_thread(client, chat_uuid),
                signal_listener_thread(client, chat_uuid),
            )
        else:
            await asyncio.gather(
                run_execution_client(client, chat_uuid),
                heartbeat_thread(client, chat_uuid),
                signal_listener_thread(client, chat_uuid),
            )


# Helper functions
async def create_chat(api_key: str, base_api_url: str) -> str | None:
    try:
        async with RemoteExecutionClient.session(
            api_key, base_api_url, os.getcwd()
        ) as client:
            chat = await client.create_chat()
            return chat.chat_uuid
    except httpx.ConnectError as e:
        click.secho(f"Error: {e}", fg="red")
        return None

==> exponent_run-0.0.36-py3-none-any.whl: exponent/commands/common.py <==
import _thread
import asyncio
import logging
import os
import sys
from collections.abc import Coroutine
from typing import Any

import click
import httpx
from dotenv import load_dotenv
from exponent.commands.utils import ConnectionTracker
from exponent.core.config import (
    Settings,
    get_settings,
)
from exponent.core.remote_execution.client import RemoteExecutionClient
from exponent.core.remote_execution.exceptions import ExponentError
from exponent.core.remote_execution.types import (
    RemoteExecutionRequestType,
    RemoteExecutionResponse,
    UseToolsConfig,
)

load_dotenv()


def set_log_level() -> None:
    settings = get_settings()
    logging.basicConfig(level=getattr(logging, settings.log_level), stream=sys.stdout)


def redirect_to_login(settings: Settings, cause: str = "detected") -> None:
    if inside_ssh_session():
        click.echo(f"No API Key {cause}, run 'exponent login --key <API-KEY>'")
    else:
        click.echo("No API Key detected, redirecting to login...")
        click.launch(f"{settings.base_url}/")


def inside_ssh_session() -> bool:
    return (os.environ.get("SSH_TTY") or os.environ.get("SSH_TTY")) is not None


def run_until_complete(coro: Coroutine[Any, Any, Any]) -> Any:
    loop = asyncio.get_event_loop()
    task = loop.create_task(coro)

    try:
        loop.run_until_complete(task)
    except KeyboardInterrupt:
        task.cancel()
        try:
            loop.run_until_complete(task)
        except asyncio.CancelledError:
            pass
    except ExponentError as e:
        click.secho(f"Encountered error: {e}", fg="red")
        click.secho(
            "The Exponent team has been notified, "
            "please try again and reach out if the problem persists.",
            fg="yellow",
        )
        sys.exit(1)


async def benchmark_mode_exit(client: RemoteExecutionClient, chat_uuid: str) -> None:
    while True:
        await asyncio.sleep(5)
        if await client.check_remote_end_event(chat_uuid):
            sys.exit(0)


async def heartbeat_thread(
    client: RemoteExecutionClient,
    chat_uuid: str,
    connection_tracker: ConnectionTracker | None = None,
) -> None:
    await client.send_heartbeats(chat_uuid, connection_tracker)


async def run_execution_client(client: RemoteExecutionClient, chat_uuid: str) -> None:
    async def handle_request(
        execution_request: RemoteExecutionRequestType,
    ) -> RemoteExecutionResponse:
        execution_response = await client.handle_request(execution_request)
        return execution_response

    await client.for_each_execution_request(chat_uuid, handle_request)


async def signal_listener_thread(client: RemoteExecutionClient, chat_uuid: str) -> None:
    await client.wait_for_disconnect_signal(chat_uuid)
    click.secho("Disconnect signaled. Exiting.")
    _thread.interrupt_main()


async def start_chat(
    client: RemoteExecutionClient,
    chat_uuid: str,
    prompt: str,
    use_tools_config: UseToolsConfig,
) -> None:
    click.secho("Starting chat...")
    await client.start_chat(chat_uuid, prompt, use_tools_config=use_tools_config)
    click.secho("Chat started. Open the link to join the chat.")


async def start_client(  # noqa: PLR0913
    api_key: str,
    base_api_url: str,
    chat_uuid: str,
    prompt: str | None = None,
    benchmark: bool = False,
    connection_tracker: ConnectionTracker | None = None,
) -> None:
    if benchmark is True and prompt is None:
        click.secho("Error: Benchmark mode requires a prompt.", fg="red")
        return

    current_working_directory = os.getcwd()

    use_tools_config = UseToolsConfig()
    async with RemoteExecutionClient.session(
        api_key, base_api_url, current_working_directory
    ) as client:
        if benchmark:
            assert prompt is not None
            await asyncio.gather(
                start_chat(
                    client, chat_uuid, prompt, use_tools_config=use_tools_config
                ),
                run_execution_client(client, chat_uuid),
                benchmark_mode_exit(client, chat_uuid),
                heartbeat_thread(client, chat_uuid, connection_tracker),
                signal_listener_thread(client, chat_uuid),
            )
        elif prompt:
            await asyncio.gather(
                start_chat(
                    client, chat_uuid, prompt, use_tools_config=use_tools_config
                ),
                run_execution_client(client, chat_uuid),
                heartbeat_thread(client, chat_uuid, connection_tracker),
                signal_listener_thread(client, chat_uuid),
            )
        else:
            await asyncio.gather(
                run_execution_client(client, chat_uuid),
                heartbeat_thread(client, chat_uuid, connection_tracker),
                signal_listener_thread(client, chat_uuid),
            )


# Helper functions
async def create_chat(api_key: str, base_api_url: str) -> str | None:
    try:
        async with RemoteExecutionClient.session(
            api_key, base_api_url, os.getcwd()
        ) as client:
            chat = await client.create_chat()
            return chat.chat_uuid
    except (httpx.ConnectError, ExponentError) as e:
        click.secho(f"Error: {e}", fg="red")
        return None
